__author__ = 'Sergei F. Kliver'

from KRATER.Routines.AIndexRoutines import AIndexRoutines
from KRATER.Routines.JellyfishRoutines import JellyfishRoutines

AIndexRoutines = AIndexRoutines()
JellyfishRoutines = JellyfishRoutines()
